/*
 * =====================================================================================
 *
 *       Filename:  startup_stm32f4.c
 *
 *    Description:  Startup code for STM32F4 MCU
 *                  Based on http://electronics.stackexchange.com/questions/224618/minimal-code-required-to-startup-a-stm32f4
 *
 *        Version:  1.0
 *        Created:  12/11/2016 02:54:57 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *   Organization:
 *
 * =====================================================================================
 */

#include "stdint.h"
#include "stm32f401xe.h"
/*#include "stm32f4xx.h"*/
/*#include "stm32f4xx_hal_gpio.h"*/
/*#include "stm32f4xx_hal_flash.h"*/


/** Main program entry point. */
extern int main(void);

/** Exit system call. */
extern void _exit(int code);

extern unsigned int __textdata__;
extern unsigned int __data_start__;
extern unsigned int __data_end__;
extern unsigned int __bss_start__;
extern unsigned int __bss_end__;

/** Initializes the data section. */
static void __attribute__((always_inline)) __initialize_data (unsigned int* from,
        unsigned int* region_begin, unsigned int* region_end);

/** Initializes the BSS section. */
static void __attribute__((always_inline)) __initialize_bss (unsigned int* region_begin,
        unsigned int* region_end);

/** Start-up code. */
void __attribute__ ((section(".after_vectors"), noreturn, used)) _start(void);


void _start (void)
{
    //Before switching on the main oscillator and the PLL,
    //and getting to higher and dangerous frequencies,
    //configuration of the flash controller is necessary.

    //Enable the flash prefetch buffer. Can be achieved when CCLK
    //is lower than 24MHz.
    /*FLASH_PrefetchBufferCmd(1);*/
    /* Enable or disable the Prefetch Buffer */
    /*if(NewState != DISABLE)*/
    /*{*/
    FLASH->ACR |= FLASH_ACR_PRFTEN;
    /*}*/
    /*else*/
    /*{*/
    /*FLASH->ACR &= (~FLASH_ACR_PRFTEN);*/
    /*}*/

    //Set latency to 2 clock cycles. Necessary for setting the clock
    //to the maximum 72MHz.
    /*FLASH_SetLatency(2);*/

    /* Perform Byte access to FLASH_ACR[8:0] to set the Latency value */
    *(__IO uint8_t *)FLASH_ACR_BYTE0_ADDRESS = (uint8_t)2;


    // Initialize hardware right after configuring flash, to switch
    //clock to higher frequency and have the rest of the
    //initializations run faster.
    //
    //Testing to let OpenOCD set these instead
    /*SystemInit();*/


    // Copy the DATA segment from Flash to RAM (inlined).
    __initialize_data(&__textdata__, &__data_start__, &__data_end__);

    // Zero fill the BSS section (inlined).
    __initialize_bss(&__bss_start__, &__bss_end__);


    //Core is running normally, RAM and FLASH are initialized
    //properly, now the system must be fully functional.

    //Update the SystemCoreClock variable.
    /*SystemCoreClockUpdate();*/


    // Call the main entry point, and save the exit code.
    int code = main();


    //Main should never return. If it does, let the system exit gracefully.
    _exit (code);

    // Should never reach this, _exit() should have already
    // performed a reset.
    while(1);
}

static inline void __initialize_data (unsigned int* from, unsigned int* region_begin,
                                      unsigned int* region_end)
{
    // Iterate and copy word by word.
    // It is assumed that the pointers are word aligned.
    unsigned int *p = region_begin;
    while (p < region_end)
    {
        *p++ = *from++;
    }
}

static inline void __initialize_bss (unsigned int* region_begin, unsigned int* region_end)
{
    // Iterate and clear word by word.
    // It is assumed that the pointers are word aligned.
    unsigned int *p = region_begin;
    while (p < region_end)
    {
        *p++ = 0;
    }
}
